﻿using Kivii.Finances.Entities;
using Kivii.Linq;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances
{
    public static class ApplyExtension
    {
        public static void ExcuteCompleted(this List<InvoiceApply> allApplys, IDbConnection conn = null)
        {
            if (allApplys.IsNullOrEmpty()) return;
            var applys = allApplys.Where(o => o.AmountInvoice < o.Amount).ToList();
            if (applys.IsNullOrEmpty()) return;

            var applyKvids = applys.ConvertAll(p => p.Kvid);
            bool useTransaction = conn == null;//是否启用事务，如果外部未传入conn,启用内部事务
            if (conn == null) conn = KiviiContext.GetOpenedDbConnection<InvoiceApply>();
            var queryInvoices = conn.From<Invoice>();
            queryInvoices.Where(o => o.OffsetKvid == Guid.Empty && o.RootKvid == o.Kvid && Sql.In(o.ApplyKvid, applyKvids));
            queryInvoices.Select(o => new { o.Kvid, o.Amount, o.ApplyKvid });
            var invoices = conn.Select(queryInvoices);
            if (invoices.IsNullOrEmpty()) return;
            var relations = conn.Select<InvoiceApply>(o => o.OffsetKvid == Guid.Empty && o.OperateType == InvoiceApplyType.Related && Sql.In(o.ParentKvid, applyKvids));

            List<InvoiceApply> preUpdates = new List<InvoiceApply>();
            foreach (var apply in applys)
            {
                if (apply.AmountInvoice == apply.Amount) continue;
                var currentInvoices = invoices.Where(o => o.ApplyKvid == apply.Kvid).ToList();
                if (currentInvoices.IsNullOrEmpty()) continue;
                var amountSum = currentInvoices.Sum(o => o.Amount);
                List<InvoiceApply> applyRelations = null;
                if (!relations.IsNullOrEmpty()) applyRelations = relations.Where(o => o.ParentKvid == apply.Kvid).ToList();
                if (amountSum != apply.Amount) throw new Exception($"{apply.PayerName}:发票金额{currentInvoices.Sum(o => o.Amount)}和申请金额{apply.Amount}不一致");
                apply.AmountInvoice = amountSum;
                apply.AddOnlyProperties(o => o.AmountInvoice);
                if (apply.AmountInvoice == apply.Amount)
                {
                    apply.Status = (int)InvoiceApplyStatus.Completed;
                    apply.AddOnlyProperties(o => o.Status);
                }
                preUpdates.Add(apply);

                if (!applyRelations.IsNullOrEmpty())
                {
                    foreach (var item in applyRelations)
                    {
                        item.AmountInvoice = item.Amount;
                        item.AddOnlyProperties(o => o.AmountInvoice);
                        if (item.AmountInvoice == item.Amount)
                        {
                            item.Status = (int)InvoiceApplyStatus.Completed;
                            item.AddOnlyProperties(o => o.Status);
                        }
                        preUpdates.Add(item);
                    }
                }
            }
            if (preUpdates.IsNullOrEmpty()) return;

            IDbTransaction trans = null;//事务，如果外部来了Connection，不生成事务
            if (useTransaction) trans = conn.OpenTransaction();

            try
            {
                if (!preUpdates.IsNullOrEmpty())
                {
                    foreach (var item in preUpdates)
                    {
                        if (!item.OnlyProperties.IsNullOrEmpty()) conn.UpdateOnly(item);
                    }
                }
                trans?.Commit();
            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }

        }
    }
}
